#include <fstream>
#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <cstdlib>
#include "eosform.h"

map<int,string> LoadNuclei(string fname){
	map<int,string> NucleiNames;
	ifstream IFILE("elements.txt");
	if(IFILE.bad()) {
		cerr << "WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING\n";
		cerr << ">>> couldn't load elements.txt\n";
		cerr << ">>> this is not necessarily bad \n>>> but will result in ugly output \n>>> in case your data contains nuclei\n\n";
	}
	while(!IFILE.eof()){
		int number;
		string name;
		IFILE >> number >> name;
		// cerr << number << " " << name << endl;
		NucleiNames[number]=name;
	}
	return NucleiNames;
}

string ElementName(int ZNumber, map<int,string> Nuclei){
	string NName=Nuclei[ZNumber];
	if(NName.size()==0){
		stringstream ss;//create a stringstream
		ss << ZNumber;//add number to the stream
		NName = ss.str();//return a string with the contents of the stream
	};
	return NName;
}

string NameParticle(int ParticleIdx, map<int,particle> ParticleMap, map<int,string> NucleiNames){
	string name;
	if(ParticleIdx<1000){
		name.assign(ParticleMap[ParticleIdx].sTexString);
	}
	else{
		stringstream ss;
		ss << ParticleIdx;
		name=ss.str();
		int A = (int)(ParticleIdx/1000);
		string Zstr(name,name.length()-3,3);
		int Z; // = atoi(Zstr.data());
		sscanf(Zstr.data(), "%d", &Z);

		string EName=ElementName(Z,NucleiNames);
		stringstream res;
		res << "${}_{" << A << "}^{" << Z << "}$"<< EName;
		name = res.str();
	}
	if(name.size()==0) name.assign("not indexed in CompOSE!");
	return name;
}

void FillMicroQuantities(map<int,string> &mMQMap){
	mMQMap[40]="Landau effective mass divided by particle mass $m_i^L/m_i$";
	mMQMap[41]="Dirac effective mass divided by particle mass $m_i^D/m_i$";
	mMQMap[50]="non-relativistic single-particle potential $U_i$ [MeV]";
	mMQMap[51]="relativistic vector self-energy $V_i$ [MeV]";
	mMQMap[52]="relativistic scalar self-energy $S_i$ [MeV]";
	mMQMap[60]="pairing gap $\\Delta_i$ [MeV]";
}

string MicroQuant(int iMQIndex, map<int,string> mMQMap){
	int PartIndex=(int)(iMQIndex/1000);
	int QuanIndex=iMQIndex-PartIndex*1000;
	return mMQMap[QuanIndex];
}

string MicroParti(int iMQIndex,map<int,particle> ParticleMap, map<int,string> NucleiNames){
	int ParticleIdx=(int)(iMQIndex/1000);
	return NameParticle(ParticleIdx, ParticleMap, NucleiNames);
}

void FillParticleMap(map<int,particle> &mPMap){
	mPMap[10]	= particle(BARYON,	"neutron",	"n",			1,	0,	0,	0,	0,	10);
	mPMap[11]	= particle(BARYON,	"proton",	"p",			1,	0,	1,	0,	0,	11);
	mPMap[20]	= particle(BARYON,	"delta-",	"$\\Delta^-$",		1,	0,	-1,	0,	0,	20);
	mPMap[21]	= particle(BARYON,	"delta0",	"$\\Delta^0$",		1,	0,	0,	0,	0,	21);
	mPMap[22]	= particle(BARYON,	"delta+",	"$\\Delta^+$",		1,	0,	0,	0,	0,	22);
	mPMap[23]	= particle(BARYON,	"delta++",	"$\\Delta^{++}$",	1,	0,	0,	0,	0,	23);
	mPMap[100]	= particle(BARYON,	"lambda",	"$\\Lambda$",		1,	-1,	0,	0,	0,	100);
	mPMap[110]	= particle(BARYON,	"sigma-",	"$\\Sigma^-$$",		1,	-1,	-1,	0,	0,	110);
	mPMap[111]	= particle(BARYON,	"sigma0",	"$\\Sigma^0$",		1,	-1,	0,	0,	0,	111);
	mPMap[112]	= particle(BARYON,	"sigma+",	"$\\Sigma^+$",		1,	-1,	+1,	0,	0,	112);
	mPMap[120]	= particle(BARYON,	"xi-",		"$\\Xi^-$",		1,	-2,	-1,	0,	0,	120);
	mPMap[121]	= particle(BARYON,	"xi0",		"$\\Xi^0$",		1,	-2,	0,	0,	0,	121);

	mPMap[200]	= particle(MESON,	"omega",	"$\\omega$",		0,	0,	0,	0,	0,	200);
	mPMap[210]	= particle(MESON,	"sigma",	"$\\sigma$",		0,	0,	0,	0,	0,	210);
	mPMap[220]	= particle(MESON,	"eta",		"$\\eta$",			0,	0,	0,	0,	0,	210);
	mPMap[300]	= particle(MESON,	"rho-",		"$\\rho^-$",		0,	0,	-1,	0,	0,	300);
	mPMap[301]	= particle(MESON,	"rho0",		"$\\rho^0$",		0,	0,	0,	0,	0,	301);
	mPMap[302]	= particle(MESON,	"rho+",		"$\\rho^+$",		0,	0,	+1,	0,	0,	302);
	mPMap[310]	= particle(MESON,	"delta-",	"$\\delta^-$",		0,	0,	-1,	0,	0,	310);
	mPMap[311]	= particle(MESON,	"delta0",	"$\\delta^0$",		0,	0,	0,	0,	0,	311);
	mPMap[312]	= particle(MESON,	"delta+",	"$\\delta^+$",		0,	0,	+1,	0,	0,	312);
	mPMap[320]	= particle(MESON,	"pion-",	"$\\pi^-$",		0,	0,	-1,	0,	0,	320);
	mPMap[321]	= particle(MESON,	"pion0",	"$\\pi^0$",		0,	0,	0,	0,	0,	321);
	mPMap[322]	= particle(MESON,	"pion+",	"$\\pi^+$",		0,	0,	+1,	0,	0,	322);
	mPMap[400]	= particle(MESON,	"K-",		"K$^-$$",			0,	-1,	-1,	0,	0,	400);
	mPMap[401]	= particle(MESON,	"K0",		"K$^0$",			0,	1,	0,	0,	0,	401);
	mPMap[402]	= particle(MESON,	"K0",		"K$^0$",			0,	-1,	0,	0,	0,	402);
	mPMap[403]	= particle(MESON,	"K+",		"K$^+$",			0,	1,	+1,	0,	0,	403);

	mPMap[500]	= particle(QUARK,	"up",		"u",			1,	0,	2,	0,	0,	500);
	mPMap[501]	= particle(QUARK,	"down",		"d",			1,	0,	-1,	0,	0,	501);
	mPMap[502]	= particle(QUARK,	"strange",	"s",			1,	-1,	-1,	0,	0,	502);

	mPMap[0]	= particle(LEPTON,	"electron",	"e$^-$",			0,	0,	-1,	1,	0,	0);
	mPMap[1]	= particle(LEPTON,	"muon",		"$\\mu^-$",		0,	0,	-1,	0,	1,	1);
}

int main(void){
	// define and initialize map of known particle species
	map<int,particle> mPMap;
	FillParticleMap(mPMap);

	map<int,string> mMQMap;
	FillMicroQuantities(mMQMap);


	// Load element names
	map<int,string> NucleiMap = LoadNuclei("elements.txt");

	// predefine all later used and unused variables
	string sMandat("{\\color{red}{mandatory}}");
	string sManually("{\\color{red}{fill manually}}");
	string sToBeDefined("{\\color{red}{tbd}}");
	string sVolunt("-");	
	string sEoSFullName=sMandat;
	string sEoSShortName=sMandat;
	string sEoSCategory=sMandat;//choose from nuclear/quark/hybrid
	string sAuthor=sMandat;
	string sAuthorAffil=sVolunt;
	string sAuthorEmail=sMandat;
	string sTableDim=sToBeDefined;
	string sTableType=sToBeDefined;
	string sGPNumber=sToBeDefined;
	string sTmin=sToBeDefined;
	string sTmax=sToBeDefined;
	string sTpts=sToBeDefined;
	string snmin=sToBeDefined;
	string snmax=sToBeDefined;
	string snpts=sToBeDefined;
	string sYmin=sToBeDefined;
	string sYmax=sToBeDefined;
	string sYpts=sToBeDefined;
	string sMMax="0";
	string sMDU="0";
	string sRMMax="0";
	string sRMOF="0";
	string sFEError="0";
	string sIEError="0";
	string snSatval="0";
	string sEZeroval="0";
	string sKval="0";
	string sKPval="0";
	string sJval="0";
	string sLval="0";
	string sKSval="0";

	int iTDDataExists=0;
	int iCoDataExists=0;
	int iMiDataExists=0;
	int iNaddQuantCompo=0;
	int iNPhases=0;
	int iNPhasesUnlisted=0;
	vector<int> ivPhaseIndex;
	int iNParticles=0;
	int iNParticlesUnlisted=0;
	vector<int> ivParticleIndex;
	int iNSets=0;
	int iNSetsUnlisted=0;
	vector<int> ivSetIndex;
	int iNMicro=0;
	int iNMicroUnlisted=0;
	vector<int> ivMicroIndex;

	string sInString;
	cout << "eosform:\nthis program assists you in creating the eos data form\n"
		<< "which will appear with your data as part of the CompOSE database\n"
		<< "(" << sCompOSEhyperref << ")" << endl << endl;


	cout << "Pressing enter skips a step. Be aware that these unspecified data\ncan (and should)  be added by hand to the generated tex-file\n\n";
	cout << "If your pre-generated report file is NOT named eos.report\nplease RENAME to eos.report it and restart this program.\n";

	cout << "\nFull name of your EoS:\n";
	getline(cin,sInString);
	if(sInString.size()!=0) {
		sEoSFullName=sInString;
	}else{
		cout << "reminder: fill this manually\n";
	}

	//cout << "\nShort name of your EoS:\n";
	//getline(cin,sInString);
	//if(sInString.size()!=0) {
	//	sEoSShortName=sInString;
	//}else{
	//	cout << "reminder: fill this manually\n";
	//}

	cout << "\nType of your EoS (hadronic,quark,hybrid) [will not be checked!]:\n";
	getline(cin,sInString);
	if(sInString.size()!=0) {
		sEoSCategory=sInString;
	}else{
		cout << "reminder: fill this manually\n";
	}

	cout << "\nYour full name (LaTeX style):\n";
	getline(cin,sInString);
	if(sInString.size()!=0) {
		sAuthor=sInString;
	}else{
		cout << "reminder: fill this manually\n";
	}

	cout << "\nYour full affiliation (LaTeX style):\n";
	getline(cin,sInString);
	if(sInString.size()!=0) {
		sAuthorAffil=sInString;
	}else{
		cout << "reminder: fill this manually\n";
	}

	cout << "\nYour email (feel free to use (at)(dot) etc.):\n";
	getline(cin,sInString);
	if(sInString.size()!=0) {
		sAuthorEmail=sInString;
	}else{
		cout << "reminder: fill this manually\n";
	}

	cout << "\n\nCONGRATULATIONS!\nThis has been the first part of information we needed.";
	cout << "Wait for the program to finish, compile the resulting tex-file and fill possible gaps.";

	// open eos.report
	ifstream EOSREP("eos.report");
	if(!EOSREP.is_open()) {
		cerr << "ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR \n";
		cerr << ">>> couldn't load eos.report\n";
		cerr << ">>> make sure it exists and is placed in the\n>>> same folder as the executable of this programm.";
		cerr << "Program terminates."<< endl;
		int iQuit;
		cin >> iQuit;
		exit(0);
	}

	// extract all information from eos.report
	string line;
	cout << "\n\nEXTRACTING INFORMATION FROM eos.report\n";
	string sDummy;
	while(getline(EOSREP, line)){
		if(line.find("# dimension and type of eos table")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sTableDim;
			iss >> sTableType;
			cout << "\nTable Dimension  :" << sTableDim;
			cout << "\nTable Type       :" << sTableType;
		};

		if(line.find("# number of grid points in temperature T, minimum temperature, maximum temperature [MeV]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sTpts;
			iss >> sTmin;
			iss >> sTmax;
			cout << "\n\nPts in T     :" << sTpts;
			cout << "\nT_min[MeV]   :" << sTmin;
			cout << "\nT_max[MeV]   :" << sTmax;
		};

		if(line.find("# number of grid points in baryon number density n_b, minimum baryon number density, maximum baryon number density [fm^-3]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> snpts;
			iss >> snmin;
			iss >> snmax;
			cout << "\n\nPts in n     :" << snpts;
			cout << "\nn_min[1/fm^3]:" << snmin;
			cout << "\nn_max[1/fm^3]:" << snmax;
		};

		if(line.find("# number of grid points in charge fraction Y_q, minimum charge fraction, maximum charge fraction [dimensionless]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sYpts;
			iss >> sYmin;
			iss >> sYmax;
			cout << "\n\nPts in Y     :" << sYpts;
			cout << "\nY_min        :" << sYmin;
			cout << "\nY_max        :" << sYmax;
		};

		if(line.find("# thermodynamical data exist (1) or not (0)")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sDummy;
			cout << "\n\neos.thermo ";
			if(sDummy.find("1")!=string::npos) {
				cout << "exists.";
				iTDDataExists=1;
			}
			else{
				cout << "is missing.";
				iTDDataExists=0;
			}
		};

		if(line.find("# composition data exist (1) or not (0)")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sDummy;
			cout << "\neos.compo  ";
			if(sDummy.find("1")!=string::npos) {
				cout << "exists.";
				iCoDataExists=1;
			}
			else{
				cout << "is missing.";
				iCoDataExists=0;
			}
		};

		if(line.find("# microscopic data exist (1) or not (0)")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sDummy;
			cout << "\neos.micro  ";
			if(sDummy.find("1")!=string::npos) {
				cout << "exists.";
				iMiDataExists=1;
			}
			else{
				cout << "is missing.";
				iMiDataExists=0;
			}
		};

		if(line.find("# total number of table entries")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sGPNumber;
			cout << "\n\ntotal nr of grid points in eos.thermo : " << sGPNumber;
		};

		if(line.find("# number of additional quantities in eos.thermo")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> 	iNaddQuantCompo;
			cout << "\n\nnr. of additional quantities in eos.thermo : " << iNaddQuantCompo;
		};

		if(line.find("# number of phases, number of not listed phases")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> iNPhases;
			iss >> iNPhasesUnlisted;
			cout << "\n\nnr. of phases listed/unlisted : " << iNPhases << "/" << iNPhasesUnlisted;
		};

		if(line.find("# indices of phases ")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			int idummy;
			while(iss >> idummy){
				ivPhaseIndex.push_back(idummy);
			}
			cout << "\nphase indices :";
			for(int i = 0; i<ivPhaseIndex.size();i++){
				cout << " " << ivPhaseIndex[i];
			}
		};

		if(line.find("# number of particles, number of not listed particles")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> iNParticles;
			iss >> iNParticlesUnlisted;
			cout << "\n\nnr. of particles listed/unlisted : " << iNParticles << "/" << iNParticlesUnlisted;
		};

		if(line.find("# indices of particles (see manual)")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);

			int idummy;
			while(iss >> idummy){
				ivParticleIndex.push_back(idummy);
			}
			cout << "\nparticle indices :";
			for(int i = 0; i<ivParticleIndex.size();i++){
				cout << " \n" << ivParticleIndex[i] << "\t=" << NameParticle(ivParticleIndex[i],mPMap,NucleiMap);
			}
		};

		if(line.find("# number of particle sets, number of not listed particle sets")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> iNSets;
			iss >> iNSetsUnlisted;
			cout << "\n\nnr. of particle sets listed/unlisted : " << iNSets << "/" << iNSetsUnlisted;
		};

		if(line.find("# indices of particle sets (see manual)")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);

			int idummy;
			while(iss >> idummy){
				ivSetIndex.push_back(idummy);
			}
			cout << "\nparticle set indices :";
			for(int i = 0; i<ivSetIndex.size();i++){
				cout << " \n" << ivSetIndex[i] ;
			}

		};

		if(line.find("# number of quantities in eos.micro, number of not listed quantities")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> iNMicro;
			iss >> iNMicroUnlisted;
			cout << "\n\nnr. of quantities in eos.micro listed/unlisted : " << iNMicro << "/" << iNMicroUnlisted;
		};

		if(line.find("# indices of quantities (see manual)")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			int idummy;
			while(iss >> idummy){
				ivMicroIndex.push_back(idummy);
			}
			cout << "\nindices of quantities in eos.micro:";
			for(int i = 0; i<ivMicroIndex.size();i++){
				int iMQIndex = ivMicroIndex[i];
				string quantname = MicroQuant(iMQIndex, mMQMap);
				string particle  = MicroParti(iMQIndex, mPMap,NucleiMap);
				cout << " \n" << ivMicroIndex[i] << " : " << quantname << " of " << particle;
			}
		};

		if(line.find("# saturation density of symmetric nuclear matter [fm^-3]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> snSatval;
			cout << "\n\nsaturation density of symmetric nuclear matter [fm^-3]\n" << snSatval;
		};

		if(line.find("# binding energy B_sat per baryon at saturation [MeV]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sEZeroval;
			cout << "\n\nbinding energy B_sat per baryon at saturation [MeV] \n" <<sEZeroval;
		};

		if(line.find("# incompressibility K [MeV]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sKval;
			cout << "\n\nincompressibility K [MeV] \n" <<sKval;
		};

		if(line.find("# skewness K^prime [MeV]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sKPval;
			cout << "\n\nskewness K^prime [MeV] \n" <<sKPval;
		};

		if(line.find("# symmetry energy J [MeV]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sJval;
			cout << "\n\nsymmetry energy J [MeV] \n" <<sJval;
		};

		if(line.find("# symmetry energy slope parameter L [MeV]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sLval;
			cout << "\n\nsymmetry energy slope parameter L [MeV]\n" <<sLval;
		};

		if(line.find("# symmetry incompressibility K_sym [MeV]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sKSval;
			cout << "\n\nsymmetry incompressibility K_sym [MeV] \n" <<sKSval;
		};

		if(line.find("# maximum relative error in free energy per baryon")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sFEError;
			cout << "\n\nmaximum relative error in free energy per baryon \n" <<sFEError;
		};

		if(line.find("# maximum relative error in internal energy per baryon")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sIEError;
			cout << "\n\nmaximum relative error in internal energy per baryon \n" <<sIEError;
		};

		if(line.find("# Maximum mass [M_sol]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sMMax;
			cout << "\n\nMaximum mass [M_sol] \n" <<sMMax;
		};

		if(line.find("# Radius at maximum mass [km]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sRMMax;
			cout << "\n\nRadius at maximum mass [km] \n" <<sRMMax;
		};

		if(line.find("# Radius at 1.4 solar masses [km]")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sRMOF;
			cout << "\n\nRadius at 1.4 solar masses [km] \n" << sRMOF;
		};

		if(line.find("# Critical mass for direct URCA on protons [M_sol]\n")!=string::npos){
			getline(EOSREP, line);
			istringstream iss(line);
			iss >> sMDU;
			cout << "\n\nCritical mass for direct URCA on protons [M_sol] \n" <<sMDU;
		};


	}


	// THIS IS THE TEMPLATE
	ifstream FTEMP("eosformtemplate.tex");
	if(!FTEMP.is_open()) {
		cerr << "ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR \n";
		cerr << ">>> couldn't load eosformtemplate.tex\n";
		cerr << ">>> make sure it exists and is placed in the\n>>> same folder as the executable of this programm.";
		cerr << "Program terminates."<< endl;
		int iQuit;
		cin >> iQuit;
		exit(0);
	}


	// THIS IS THE OUTPUT
	string sTexFileName="datasheet.tex";
	ofstream OFILE(sTexFileName.data());

	while(getline(FTEMP, line)){
		bool bFoundSome=false;
		if(line.find("newcommand{\\sEoSFullName}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sEoSFullName}{"<<sEoSFullName<<"}\n";
		}

		//if(line.find("newcommand{\\sEoSShortName}")!=string::npos){
		//	bFoundSome=true;
		//	OFILE << "\\newcommand{\\sEoSShortName}{"<< sEoSShortName <<"}\n";
		//}

		if(line.find("newcommand{\\sEoSCategory}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sEoSCategory}{"<< sEoSCategory <<"}\n";
		}

		if(line.find("newcommand{\\sAuthor}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sAuthor}{"<< sAuthor<<"}\n";
		}

		if(line.find("newcommand{\\sAuthorAffil}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sAuthorAffil}{"<< sAuthorAffil<<"}\n";
		}

		if(line.find("newcommand{\\sAuthorEmail}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sAuthorEmail}{"<< sAuthorEmail<<"}\n";
		}

		if(line.find("newcommand{\\sTableDim}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sTableDim}{"<< sTableDim<<"}\n";
		}

		if(line.find("newcommand{\\sTableType}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sTableType}{"<< sTableType<<"}\n";
		}

		if(line.find("newcommand{\\sGPNumber}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sGPNumber}{"<< sGPNumber<<"}\n";
		}

		if(line.find("newcommand{\\sTmin}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sTmin}{"<< sTmin<<"}\n";
		}

		if(line.find("newcommand{\\sTmax}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sTmax}	{"<< sTmax<<"}\n";
		}

		if(line.find("newcommand{\\sTpts}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sTpts}{"<<sTpts <<"}\n";
		}

		if(line.find("newcommand{\\snmin}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\snmin}	{"<< snmin<<"}\n";
		}

		if(line.find("newcommand{\\snmax}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\snmax}	{"<< snmax<<"}\n";
		}

		if(line.find("newcommand{\\snpts}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\snpts}	{"<< snpts<<"}\n";
		}

		if(line.find("newcommand{\\sYmin}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sYmin}{"<< sYmin<<"}\n";
		}

		if(line.find("newcommand{\\sYmax}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sYmax}{"<< sYmax<<"}\n";
		}

		if(line.find("newcommand{\\sYpts}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sYpts}	{"<< sYpts<<"}\n";
		}

		if(line.find("newcommand{\\sMMax}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sMMax}{"<<sMMax <<"}\n";
		}

		if(line.find("newcommand{\\sMDU}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sMDU}{"<< sMDU<<"}\n";
		}

		if(line.find("newcommand{\\sRMMax}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sRMMax}{"<<sRMMax <<"}\n";
		}

		if(line.find("newcommand{\\sRMOF}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sRMOF}	{"<< sRMOF<<"}\n";
		}

		if(line.find("newcommand{\\sFEError}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sFEError}{"<<sFEError <<"}\n";
		}

		if(line.find("newcommand{\\sIEError}	")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sIEError}	{"<<sIEError <<"}\n";
		}

		if(line.find("newcommand{\\snSatval}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\snSatval}{"<< snSatval<<"}\n";
		}

		if(line.find("newcommand{\\sEZeroval}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sEZeroval}{"<< sEZeroval<<"}\n";
		}

		if(line.find("newcommand{\\sKval}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sKval}{"<< sKval<<"}\n";
		}

		if(line.find("newcommand{\\sKPval}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sKPval}{"<< sKPval<<"}\n";
		}

		if(line.find("newcommand{\\sJval}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sJval}{"<< sJval<<"}\n";
		}

		if(line.find("newcommand{\\sLval}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sLval}{"<<sLval <<"}\n";
		}

		if(line.find("newcommand{\\sKSval}")!=string::npos){
			bFoundSome=true;
			OFILE << "\\newcommand{\\sKSval}{"<< sKSval<<"}\n";
		}


//// DONE WITH THE INITIAL CONSTANTS

		// TO BE COMPLETED
		if(line.find("% EOS.THERMO START")!=string::npos){
			OFILE << "\\subsubsection{additional quantities in eos.thermo}\n";
			if(iNaddQuantCompo==0){
				OFILE << "none defined\n";
			}
			else{
				OFILE << "please contact thomas.klaehn@gmail.com. something is missing!\n";
			}
		}

		if(line.find("% EOS.COMPO START")!=string::npos){
			OFILE << "{\\bf eos.compo}  : ";
			if(!iCoDataExists) OFILE << "not ";
			OFILE << "available\n";
			if(iCoDataExists){
				OFILE << "\\begin{center}\n";
				OFILE << "\\tablefirsthead{index & particle\\\\}\n";
				OFILE << "\\tablehead{index & quantity\\\\}\n";
				OFILE << "\\tabletail{& - continued on next page - \\\\}\n";
				OFILE << "\\tablelasttail{&- end of table - \\\\}\n";
				OFILE << "\\begin{supertabular}{c|l}\n";
				for(int i=0; i<ivParticleIndex.size();i++){
					int index = ivParticleIndex[i];
					string name = NameParticle(ivParticleIndex[i],mPMap,NucleiMap);
					OFILE << index << " & " << name << "\\\\\n" ;
				}
				OFILE << "\\end{supertabular}\n";
				OFILE << "\\end{center}\n";
			}
			OFILE << "further particle sets are ";
			if(iNSets==0){
				OFILE << "not ";
			};
			OFILE << "defined.\n";
			if(iNSets>0){
				OFILE << "\\begin{center}\n";
				OFILE << "\\tablefirsthead{index & description  \\\\}\n";
				OFILE << "\\tablehead{index & description\\\\}\n";
				OFILE << "\\tabletail{&- continued on next page - \\\\}\n";
				OFILE << "\\tablelasttail{&- end of table - \\\\}\n";
				OFILE << "\\begin{supertabular}{c|l}\n";
				for(int i=0;i<ivSetIndex.size();i++){
					OFILE << ivSetIndex[i] << " & PLEASE DESCRIBE THE SET \\\\" << endl;
				}
				OFILE << "\\end{supertabular}\n";
				OFILE << "\\end{center}\n";
			}
		}

		if(line.find("% EOS.MICRO START")!=string::npos){
			OFILE << "{\\bf eos.micro}  : ";
			if(!iMiDataExists) OFILE << "not ";
			OFILE << "available\n";
			if(iMiDataExists){
				OFILE << "\\begin{center}\n";
				OFILE << "\\tablefirsthead{index  & quantity & particle\\\\}\n";
				OFILE << "\\tablehead{index & quantity & particle\\\\}\n";
				OFILE << "\\tabletail{& - continued on next page - & \\\\}\n";
				OFILE << "\\tablelasttail{&- end of table -& \\\\}\n";
				OFILE << "\\begin{supertabular}{c|cl}\n";
				for(int i=0; i<ivMicroIndex.size();i++){
					int iMQIndex = ivMicroIndex[i];
					string quantname = MicroQuant(iMQIndex, mMQMap);
					string particle  = MicroParti(iMQIndex, mPMap,NucleiMap);
					OFILE << iMQIndex << " & " << quantname << " & " << particle << "\\\\\n" ;
				}
				OFILE << "\\end{supertabular}\n";
				OFILE << "\\end{center}\n";
			}
		}

		if(line.find("% PHASES START")!=string::npos){
			if(iNPhases!=0){
				for(int i=0;i<ivPhaseIndex.size();i++){
					OFILE << "\\\\{\\bf PHASE INDEX \\#" << ivPhaseIndex[i] << ":}\\\\\n";
					OFILE << "add a brief description\n";
				}
			}
		}
		//if(line.find("")!=string::npos){
		//	bFoundSome=true;
		//	OFILE << "{"<< <<"}\n";
		//}


		//if nothing has been triggered we just take the input as it is
		if(!bFoundSome) OFILE << line << "\n";
	}


	//string sHtmlFileName=sEoSNameNoSuffix+".html";
	//ofstream HFILE(sHtmlFileName.data());
	//cout << "created " << sEoSNameNoSuffix << ".tex and " << sEoSNameNoSuffix << ".html" << endl;

	//for(int i=0;i<400;i++){
	//	cerr << "|" << ElementName(i,NucleiMap) << "|" << endl;
	//};

	cout << "\n\nYour data sheet has been generated.\nPlease enter missing information.\nThe datasheet is part of the data!\n";
	getline(cin,sInString);

	// cleaning up
	FTEMP.close();
	EOSREP.close();
	OFILE.close();
	return 1;
}